/*
 * VirtualFight.java
 *
 * Created on June 29, 2006, 1:56 PM
 *
 *  Virtual Fight. A programming game.
 *  Copyright (C) 2006  Aidin Gharibnavaz.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package virtualfight;

/**
 * @author Aidin Gharibnavaz
 *         (aidin.vf@gmail.com)
 */

import virtualfight.map.Map;
import virtualfight.group.Charachter;
import virtualfight.group.Group;
import virtualfight.network.SimulationServer;
import virtualfight.GUI.guiHandler;

import java.io.FileNotFoundException;

/*
 * We are start out program here, creating map,
 * groups, and a simulation server.
 * We are use simulation server to send messages
 * to the clients.
 */
public class VirtualFight {
    
    private Map map;
    private Group[] groups;
    private int maxGroups;
    private SimulationServer server;
    private guiHandler  GUIs;
    private boolean allConnected;
    private String  shootRequested="";
    
    public VirtualFight(String mapAddress,int maxGroups){
        
        //Create new map from map file in mapAddress.
        //If mapAddress dosen't exist, program will be terminate.
        map=new Map(mapAddress);
        this.maxGroups=maxGroups;
        //If the maxGroup arg is greater than default maxGroup in map.
        if (this.maxGroups>map.maxGroups){
            System.out.println("Invalid argument.");
            System.out.println("Number of groups is greater than default" +
                    "max group in the map file.");
            System.exit(1);
        }
        
        //If everything is ok, print NO WARRANTY message.
        System.out.println("Virtual Fight, Copyright (C) 2006 Aidin Gharibnavaz ");
        System.out.println("<aidin.vf@gmail.com>");
        System.out.println("If you have any questions, "+
                "comments, or suggestions about this program, please feel free " +
                "to email them to me.");
        System.out.println("You can always find out the latest news " +
                "about Virtual Fight from its website at " +
                "http://virtualfight.sourceforge.net/");
        System.out.println(" Virtual Fight comes with ABSOLUTELY NO WARRANTY; for details," +
                " see the LICENCE file. This is free software, and you are " +
                "welcome to redistribute it under certain conditions; for details," +
                " see the LICENCE file");
        System.out.println();
        
        //Run guiHandler, so GUIs can connect to the server.
        GUIs=new guiHandler(this.maxGroups, this.map);
        GUIs.start();
        
        this.groups = new Group[this.maxGroups];
        //Clients don't allow to do anything until all of
        //the other clients are connected. (see 'analyzeMessage')
        allConnected=false;
        //waiting for connections.
        server =new SimulationServer(4500, this.maxGroups,this);
        System.out.println("all of the clients are connected.");
        allConnected=true;
        //create groups, and set charachter positions.
        for (int i=0;i<this.maxGroups;i++){
            this.groups[i]=new Group(i, map);
            this.server.client[i].sendMessage("READY");
            this.groups[i].resetTurns();  //Tell the group that game started.
        }
        //Tell the GUIs that game started.
        GUIs.updateGUIs();
    }
    
    /*
     * Analyze receved message, and do the request.
     * For example move charachter, send charachter's
     * position, and so on.
     *
     * Error messages:
     * 255  | OK.
     * 100  | Invalid command.
     */
    public void analyzeMessage(String message,int ID){
        //If Some of the clients not connected yet.
        if(!this.allConnected){
            return;
        }
        Integer chNumber;
        String  action,direction;
        String[] result;
        //messages are in a format like this:
        //n&Command&Direction
        //if there is no '&' in the message, it will exit
        //and return 'Invalid command' error.
        if(!message.contains("&")){
            server.client[ID].sendMessage(100);
            return;
        }
        result=message.split("&");
        chNumber=new Integer(result[0]);
        action=result[1];
        //chNumber is out of range.
        if ((chNumber>4)||(chNumber<0)){
            server.client[ID].sendMessage(100);
            return;
        }
        //(result.length<3) mean:
        //There is no direction in the message.
        if (result.length>2){
            direction=result[2];
        }else{
            direction=null;
        }
        
        //Do 'MOVE' request, and send result to the client.
        if (action.equals("MOVE")){
            //If client request 'MOVE', but direction is empty,
            //We send it an error message.
            if (direction!=null){
                this.server.client[ID].sendMessage(
                    this.groups[ID].moveCharachter(chNumber, direction));
            }else{
                this.server.client[ID].sendMessage(100);
            }
        }
        
        //We record group id and charachter number of the charachter
        //who request a shoot, in a String (shootRequested) Then after
        //a turn is completed, we proccess this String and see who
        //is killed. So if charachter A shoot to charachter B, and
        //at the same time B is moving, A can't kill B.
        if (action.equals("SHOOT")){
            if(groups[ID].isAlive(chNumber)){
                if (direction==null){
                    this.shootRequested=this.shootRequested+ID+","+chNumber+"&";
                }else{
                    this.shootRequested=this.shootRequested+ID+","+chNumber
                            +","+direction+"&";
                }
                this.groups[ID].chMoveDone(chNumber);
                this.server.client[ID].sendMessage(255);
            }else{
                this.server.client[ID].sendMessage(120);
            }
        }
        
        //Do 'POS' request, send charachter position to the client.
        if (action.equals("POS")){
            this.server.client[ID].sendMessage(
                this.groups[ID].getCharachterLocation(chNumber));
        }
        
        //Do 'DIR' request, send charachter direction to the client.
        if (action.equals("DIR")){
            this.server.client[ID].sendMessage(
                    this.groups[ID].getCharachterDirection(chNumber));
        }
        
        //Do 'SAW' request, and send result to the client.
        if (action.equals("SAW")){
            this.server.client[ID].sendMessage(
                    this.groups[ID].whatDoYouSee(chNumber));
        }
        
        //Do 'NONE' request, and send result to the client.
        if (action.equals("NONE")){
            //If client request 'NONE' with a direction.
            if (direction!=null){
                this.server.client[ID].sendMessage(
                    this.groups[ID].changeCharDirection(chNumber,direction));
            //If client request 'NONE' without a direction.    
            }else{
                //Do nothing, stay there and watch!
                this.groups[ID].chMoveDone(chNumber);
                this.server.client[ID].sendMessage(255);
            }
        }
        
        //If all of the clients did their moves,
        //turn was done, and we start another turn.
        for (int i=0;i<maxGroups;i++){
            //If one of the groups not did its movement,
            //We return.
            if(!groups[i].isAllDone()){
                return;
            }
        }
        //If all of the groups did their turn.
        proccessShootRequest();
        for (int i=0;i<maxGroups;i++){
            groups[i].resetTurns();
            server.client[i].sendMessage("READY");
        }
        //Update GUIs.
        GUIs.updateGUIs();
    }
    
    //Proccess 'shootRequest' variable.
    //(see that big comment indside the analyzeMessage for more info)
    private void proccessShootRequest(){
        String[] requests;
        String[] splitedRequest;
        int[] result=null;
        
        //If no one request a shoot.
        if (this.shootRequested.equals("")){return;}
        requests=this.shootRequested.split("&");
        //Send request to the group.
        for (int i=0;i<requests.length;i++){
            splitedRequest=requests[i].split(",");
            if (splitedRequest.length==2){
                result=this.groups[new Integer(splitedRequest[0])].shoot(
                        new Integer(splitedRequest[1]),null);
            }else{
                result=this.groups[new Integer(splitedRequest[0])].shoot(
                        new Integer(splitedRequest[1]),splitedRequest[2]);
            }
            
            //check who killed who.
            if (result[0]==200){
                //################
                System.out.println("Charachter"+splitedRequest[1]+" hit nothing.");
                //If charachter hit nothing.
                this.server.client[new Integer(splitedRequest[0])].sendMessage(
                        200);
                this.server.client[new Integer(splitedRequest[0])].sendMessage(
                        splitedRequest[1]+"&HIT&NOTHING");
                        
            }else if(result[0]==100){
                //################
                System.out.println("Charachter"+splitedRequest[1]+" hit a wall");
                //If charachter hit a wall of dead body.
                this.server.client[new Integer(splitedRequest[0])].sendMessage(
                        200);
                this.server.client[new Integer(splitedRequest[0])].sendMessage(
                        splitedRequest[1]+"&HIT&WALL");
            }else{
                //number of charachter that has been killed.
                int killedChNumber;
                //################
                System.out.println("Charachter"+splitedRequest[1]+" killed charachter at "+
                        result[1]+","+result[2]+" of group"+result[0]);
                killedChNumber=this.groups[result[0]].killChAt(result[1], result[2]);
                //If charachter hit a wall.
                this.server.client[new Integer(splitedRequest[0])].sendMessage(
                        200);
                //If charachter kill one of his teammates.
                if (splitedRequest[0].equals(result[0]+"")){
                    this.server.client[new Integer(splitedRequest[0])].sendMessage(
                            splitedRequest[1]+"&HIT&OWN");
                //If charachter kill an enemy.
                }else{
                    this.server.client[new Integer(splitedRequest[0])].sendMessage(
                            splitedRequest[1]+"&HIT&ENEMY");
                }
                //Tell the group that his charachter is died.
                this.server.client[result[0]].sendMessage(
                        200);
                this.server.client[result[0]].sendMessage(
                        killedChNumber+"&KILLED");
            }
        }
        this.shootRequested="";
    }
    
    /*
     * Get args, and start program (by 
     * calling 'new VirtualFight')
     */
    public static void main(String[] args) {
        if (args.length<2){
            System.out.println("Too few arguments");
            System.out.println("Usage: virtualfight mapFile numberOfGroups");
            System.exit(1);
        }
        
        Integer maxGroups=new Integer(args[1]);
        new VirtualFight(args[0],maxGroups);
    }
    
}
