/*
 * map.java
 *
 * Created on June 29, 2006, 4:10 PM
 *
 *  Virtual Fight. A programming game.
 *  Copyright (C) 2006  Aidin Gharibnavaz.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package virtualfight.map;

/**
 * @author Aidin Gharibnavaz
 *         (aidin.vf@gmail.com)
 */

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import virtualfight.group.Group;

/*
 * 'objects' hold position of objects on the map, like
 * walls and charachters. so we can check if there are
 * an object in the specific position.
 * chs hold the starting position of the groups in the
 * map. It use only one time, when we wants to set the
 * starting position of the charachters.
 */
public class Map {
    public Integer width,height,maxGroups;
    public char[][] objects;
    public char[][] chs;
    
    public Map(String fileAddress) {
        readMap(fileAddress);
    }
    
    public void readMap(String fileAddress){
        try{
            BufferedReader mapFile=new BufferedReader(new FileReader(fileAddress));
            this.maxGroups=new Integer(mapFile.readLine());
            this.width=new Integer(mapFile.readLine());
            this.height=new Integer(mapFile.readLine());
            
            //read objects and starting positions of Charachters.
            this.objects=new char[this.height][this.width];
            this.chs=new char[this.height][this.width];
            
            int lineIndex=0;
            String line;
            while((line=mapFile.readLine())!=null){
                for(int index=0;index<line.length();index++){
                    //If there is wall or nothing.
                    if ((line.charAt(index)=='#')||(line.charAt(index)=='-')){
                        this.objects[lineIndex][index]=line.charAt(index);
                    }else{ //If there is charachter.
                        //chs hold charachters positions. In placeCharachters
                        //method, we replace objects with group number.
                        //If some groups not connect, objects group position
                        //will remain empty(-).
                        this.chs[lineIndex][index]=line.charAt(index);
                        this.objects[lineIndex][index]='-';
                    }
                }
                lineIndex++;
            }
        }catch (IOException e){
            //If file not found, print error and exit.
            e.printStackTrace();
            System.exit(1);
        }
        
    }
    
    /*
     * Insert charachter positions in the object array.
     */
    public void placeCharachters(Group group){
        int chNumber=0;
        for (int j=0;j<this.height;j++){
            for (int i=0;i<this.width;i++){
                //found position of this group in the map and
                //place charachters.
                if (chs[j][i]==group.getID().toString().charAt(0)){
                    group.setCharLocation(chNumber,j,i);
                    objects[j][i]=group.getID().toString().charAt(0);
                    chNumber=chNumber+1;
                }
            }
        }
    }
    
    /*
     * If a charachter move, position of him will be changed in
     * objects array.
     */
    public void charachterMoved(int currentX,int currentY,int newX, int newY
            ,Integer groupID){
        objects[currentX][currentY]='-';
        objects[newX][newY]=groupID.toString().charAt(0);
    }
    
    /*
     * Replace charachter number in the objects array 
     * with x (dead body).
     */
    public void insertDeadBody(int j, int i){
        this.objects[j][i]='x';
    }
    
    /*
     * Return true if a specific position is empty.
     */
    public boolean isEmpty(int j, int i){
        //##########################################
        System.out.println("obj"+j+","+i+":"+objects[j][i]);
        //##########################################
        if (objects[j][i]=='-'){
            return true;
        }
        return false;
    }
    
    /*
     * Return object positions in a String format.
     * We use it for update GUIs.
     */
    public String getObjectPositions(){
        String result="";
        for (int j=0;j<this.height;j++){
            for (int i=0;i<this.width;i++){
                if (objects[j][i]!='-'){
                    result=result+"&"+j+","+i+","+objects[j][i];
                }
            }
        }
        //Delete extra & at the begining.
        result=result.substring(1);
        return result;
    }
    
    /*
     * Return object in the specific position.
     */
    public char getObject(int j,int i){
        return this.objects[j][i];
    }
    
    /*
     * Return what is in range of a charachter.
     * It call from Group.java
     *
     * Result format is somthing like:
     * 3,4,#&4,3,O&4,5,E
     * Tow first numbers are the position of the object.
     * # mean the wall,O is our group member, E is an enemy,
     * x is a dead body.
     */
    public String whatInSight(int ylocation,int xlocation,
            String direction,Integer groupID){
        String result=""; //Hold the result.
        //Hold positions that are out of sight.
        boolean[][] outOfSight=new boolean[3][7];
        
        //By default, when direction equals to 'UP' or 'DOWN',
        //this positions are out of sight:
        outOfSight[0][0]=true;
        outOfSight[0][1]=true;
        outOfSight[1][0]=true;
        outOfSight[0][5]=true;
        outOfSight[0][6]=true;
        outOfSight[1][6]=true;
        
        if (direction.equals("UP")){
            for (int j=-3;j<0;j++){
                for (int i=-3;i<4;i++){
                    if((xlocation+i>=0)&&(xlocation+i<this.width-1)&&
                            (ylocation+j>=0)&&(!outOfSight[-(j+1)][i+3])){
                        //Add what in sight and its position to resutl.
                        if(objects[ylocation+j][xlocation+i]!='-'){
                            //If there is one of our own group members.
                            if(objects[ylocation+j][xlocation+i]==
                                    groupID.toString().charAt(0)){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",O";
                            //If there is a wall.
                            }else if(objects[ylocation+j][xlocation+i]=='#'){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",#";
                            //If there is a dead body.
                            }else if(objects[ylocation+j][xlocation+i]=='x'){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                        ",X";
                            //If there is an enemy.
                            }else{
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",E";
                            }
                        } 
                    }
                }
            }
            //Delete extra & at the beginning of the string.
            if (result!=""){
                result=result.substring(1);
            }
            return result;
        }
        
        if (direction.equals("DOWN")){
            for (int j=1;j<4;j++){
                for (int i=-3;i<4;i++){
                    if((xlocation+i>=0)&&(xlocation+i<this.width-1)&&
                            (ylocation+j<this.height-1)&&(!outOfSight[j-1][i+3])){
                        //Add what in sight and its position to resutl.
                        if(objects[ylocation+j][xlocation+i]!='-'){
                            //If there is one of our own group members.
                            if(objects[ylocation+j][xlocation+i]==
                                    groupID.toString().charAt(0)){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",O";
                            //If there is a wall.
                            }else if(objects[ylocation+j][xlocation+i]=='#'){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",#";
                            //If there is an enemy.
                            }else{
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",E";
                            }
                        } 
                    }
                }
            }
            //Delete extra & at the beginning of the string.
            if (result!=""){
                result=result.substring(1);
            }
            return result;
        }
        
        outOfSight=new boolean[7][3];
        //By default, when direction equals to 'RIGHT' or 'LEFT',
        //this positions are out of sight:
        outOfSight[0][0]=true;
        outOfSight[0][2]=true;
        outOfSight[1][0]=true;
        outOfSight[5][0]=true;
        outOfSight[6][0]=true;
        outOfSight[6][1]=true;
        
        if (direction.equals("RIGHT")){
            for (int i=1;i<4;i++){
                for (int j=-3;j<4;j++){
                    if((ylocation+j<this.height-1)&&(ylocation+j>0)&&
                            (xlocation+i<this.width-1)&&(!outOfSight[j+3][i-1])){
                        //Add what in sight and its position to resutl.
                        if(objects[ylocation+j][xlocation+i]!='-'){
                            //If there is one of our own group members.
                            if(objects[ylocation+j][xlocation+i]==
                                    groupID.toString().charAt(0)){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",O";
                            //If there is a wall.
                            }else if(objects[ylocation+j][xlocation+i]=='#'){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",#";
                            //If there is an enemy.
                            }else{
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",E";
                            }
                        } 
                    }
                }
            }
            //Delete extra & at the beginning of the string.
            if (result!=""){
                result=result.substring(1);
            }
            return result;
        }
        
        if (direction.equals("LEFT")){
            for (int i=-1;i<0;i++){
                for (int j=-3;j<4;j++){
                    //#########################
                    System.out.println(j+","+i);
                    //#########################
                    if((ylocation+j<this.height-1)&&(ylocation+j>0)&&
                            (xlocation+i<this.width-1)&&(!outOfSight[j+3][-(i+1)])){
                        //Add what in sight and its position to resutl.
                        if(objects[ylocation+j][xlocation+i]!='-'){
                            //If there is one of our own group members.
                            if(objects[ylocation+j][xlocation+i]==
                                    groupID.toString().charAt(0)){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",O";
                            //If there is a wall.
                            }else if(objects[ylocation+j][xlocation+i]=='#'){
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",#";
                            //If there is an enemy.
                            }else{
                                result=result+"&"+(ylocation+j)+","+(xlocation+i)+
                                    ",E";
                            }
                        } 
                    }
                }
            }
            //Delete extra & at the beginning of the string.
            if (result!=""){
                result=result.substring(1);
            }
            return result;
        }
        
        return result;
    }
}
